/* basecodeGLUT.c
 * Code de base permettant la cration d'une fentre
 * OpenGL en utilisant les routines GLUT.
 * Touches:
 *	ESC - Quitter
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 16 septembre 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Caractristiques de la fentre et de l'cran */
#define TITLE "Example BaseCodeGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 350	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre
#define FULLSCREEN_WIDTH 640	// largeur en mode plein cran
#define FULLSCREEN_HEIGHT 480	// hauteur en mode plein cran
#define FULLSCREEN_DEPTH 16	// bit par pixels en mode plen cran

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Fullscreen */
static bool g_fullscreen = false;	

/* Prototypes */
void init(void);
void askFullscreen(void);
void glutDisplay(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void terminateApplication(void);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	#ifdef _WIN32
	askFullscreen();
	#endif
	if (!g_fullscreen) {
		glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
		glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
		glutCreateWindow(TITLE);
	}
	init();
	glutDisplayFunc(glutDisplay);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutMainLoop();
	return 0;
}

/* Permet de passer en mode plein ecran ou en mode fenetre
 * Utilise GLUT_GAME_MODE disponible uniquement sous Windows
 */
#ifdef _WIN32
void askFullscreen(void)
{
	char gameMode[14];
	
	/* Demande  l'utilisateur s'il veut le mode plein cran */
	if (!g_fullscreen)
	g_fullscreen = (MessageBox(NULL, "Click Yes to go to full screen (Recommended)", "Start FullScreen?",
						MB_ICONQUESTION | MB_YESNO) == IDYES);
	
	
	if (g_fullscreen) {	// game mode (fullscreen)
		sprintf(gameMode, "%dx%d:%d", FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, FULLSCREEN_DEPTH);
		glutGameModeString(gameMode);
		// teste si le game mode est possible
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
			//glutSetCursor(GLUT_CURSOR_NONE);
		}
		else {
			g_fullscreen = false;
		}
	} // mode fentr
}
#endif

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
	printf("\tESC - Quitter\n");
	
	/* Initialisation d'OpenGL */
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	
	glShadeModel(GL_SMOOTH);
}
		
/* Affichage et animation */
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// 	  Position      View	   Up Vector
	gluLookAt(0, 0, 5,     0, 0, 0,     0, 1, 0);
	
	// on dessine un simple triangle
	// avec 3 sommets de couleurs diffrents
	glBegin(GL_TRIANGLES);
		glColor3f(1.0, 0.0, 0.0);	// rouge
		glVertex3f(-1.5, -1.0, 0.0);
		glColor3f(0.0, 1.0, 0.0);	// vert
		glVertex3f(1.5, -1.0, 0.0);
		glColor3f(0.0, 0.0, 1.0);	// bleu
		glVertex3f(0.0, 1.0, 0.0);
	glEnd();
	
	glFlush();
	glutSwapBuffers();
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	if (h == 0) h = 1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)w/(GLfloat)h, 0.1f, 20.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
	}
}

/* Termine l'application */
void terminateApplication(void)
{
	#ifdef _WIN32
	if (g_fullscreen) glutLeaveGameMode();
	#endif
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}



